import base64
import cloudscraper
import hashlib
import json
import math
import os
import re
import time
import yaml
from datetime import date, datetime, timedelta
from lxml import html
from dateutil.tz import tzoffset
from .logger import logger as log

def alpha_chars(s):
    return ''.join([c for c in s if c.isalpha()])

def alphanum_chars(s):
    return ''.join([c for c in s if c.isalnum()])

def camel_case(s):
    return ''.join(word.title() for word in s.split('_'))
        
def array_param(l):
    return ', '.join('?'*len(l))

def dict_subset(d, keys):
    return {key: d[key] for key in keys}

def get_checksum(s):
    return hashlib.md5(s.encode('utf-8')).hexdigest()

def get_timestamp():
    return datetime.now(tzoffset('EDT', -4*60*60)).replace(microsecond=0).isoformat()

def get_timestamp_str():
    return datetime.now().strftime("%Y_%m_%d_%H_%M_%S")

def image_to_base64(content):
    b64img = base64.b64encode(content)
    return "data:image/jpeg;base64," + b64img.decode('utf-8')

def url2filename(url):
    url = url.encode('UTF-8')
    return base64.urlsafe_b64encode(url).decode('UTF-8')

def filename2url(f):
    return base64.urlsafe_b64decode(f).decode('UTF-8')

def tree_from_file(filepath):
    with open(filepath, 'r', encoding='utf-8') as f:
        content = f.read()
    tree = html.fromstring(content)
    return tree

def optional_nonalphanum_regex(s):
    return '(?i)' + ''.join([ch if ch.isalnum() or ch == '\\' else ch + '*' for ch in re.escape(s)])

def strip_end(text, suffix):
    if suffix and text.endswith(suffix):
        return text[:-len(suffix)]
    return text

def scrape(url, filepath, overwrite=False):
    log.LogDebug(f'scrape {url}')
    if not overwrite and os.path.exists(filepath):
        with open(filepath, 'r', encoding='utf-8') as f:
            return html.fromstring(f.read())
    while True:
        scraper = cloudscraper.create_scraper()
        try:
            scraped = scraper.get(url, timeout=(3,7))
        except Exception as e:
            log.LogDebug("scrape error %s" % e)
        if scraped.status_code < 400:
            break
        else:
            log.LogDebug("HTTP error %s" % scraped.status_code)
        time.sleep(2)
    
    with open(filepath, 'wb') as f:
        f.write(scraped.content)
    return html.fromstring(scraped.content)

def scrape_image(url):
    log.LogDebug(f'scrape_image {url}')
    while True:
        scraper = cloudscraper.create_scraper()
        try:
            scraped = scraper.get(url, timeout=(3,7))
        except Exception as e:
            log.LogDebug("scrape error %s" %e )
        if scraped.status_code < 400:
            break
        else:
            log.LogDebug("HTTP error %s" % scraped.status_code)
        time.sleep(2)
    return scraped.content

PARTPATTERN = re.compile('(.*?)(?:-poster|-fanart)*(?: part(\d+))*$')

def parse_part(name):
    m = re.search(PARTPATTERN, name)
    if m:
        try:
            part = int(m.group(2))
        except:
            part = 1
        return part, m.group(1)
    else:
      return 1, name

def file_read_lines(filepath, encoding='utf8'):
    with open(filepath, 'r', encoding=encoding) as f:
        return [x.strip() for x in f.readlines()]

def file_write_lines(lines, filepath):
    with open(filepath, 'w') as f:
        for line in lines:
            f.write(str(line) + '\n')

def file_write_line_groups(groups, filepath):
    lines = []
    for group in groups:
        lines += group
        lines.append('')
    file_write_lines(lines, filepath)

def file_read_line_groups(filepath, n=2, encoding='utf8'):
    results = []
    lines = file_read_lines(filepath, encoding)
    for i in range(0, len(lines), n + 1):
        items = []
        for j in range(0, n):
            items.append(lines[i + j])
        results.append(items)
    return results

def partition(l, n):
    n = max(1, n)
    return (l[i:i+n] for i in range(0, len(l), n))

def load_json(filepath, default=None):
    if not default:
        default = {}
    dirpath = os.path.dirname(os.path.abspath(filepath))
    if not os.path.exists(dirpath):
        os.makedirs(dirpath)
    try:
        return json.load(open(filepath, encoding='utf-8'))
    except Exception as e:
        log.LogError(str(e))
        if not os.path.exists(filepath):
            if default is not None:
                with open(filepath, 'w') as f:
                    f.write(json.dumps(default, ensure_ascii=True, sort_keys=True, indent=4))
        return default

def load_versioned_json(dirpath, default=None):
    if not default:
        default = {}
    if os.path.exists(dirpath) and os.listdir(dirpath):
        latest_file = os.path.join(dirpath, os.listdir(dirpath)[-1])
    else:
        latest_file = os.path.join(dirpath, get_timestamp_str() + '.json')
    return load_json(latest_file, default)

def save_json(filepath, data, ensure_ascii=True, sort_keys=True, indent=4):
    dirpath = os.path.dirname(os.path.abspath(filepath))
    if not os.path.exists(dirpath):
        os.makedirs(dirpath)
    with open(filepath, 'w') as f:
        f.write(json.dumps(data, ensure_ascii=ensure_ascii, sort_keys=sort_keys, indent=indent))

def save_versioned_json(dirpath, data, ensure_ascii=True, sort_keys=True, indent=4):
    filepath = os.path.join(dirpath, get_timestamp_str() + '.json')
    save_json(filepath, data, ensure_ascii, sort_keys, indent)
    return filepath


def load_yaml(filepath, default=None):
    if not default:
        default = {}
    dirpath = os.path.dirname(os.path.abspath(filepath))
    if not os.path.exists(dirpath):
        os.makedirs(dirpath)
    try:
        data = yaml.load(open(filepath), Loader=yaml.FullLoader)
        return data or default
    except:
        if not os.path.exists(filepath):
            if default is not None:
                with open(filepath, 'w') as f:
                    yaml.dump(default, f, default_flow_style=False)
        return default

def load_versioned_yaml(dirpath, default=None):
    if not default:
        default = {}
    if os.path.exists(dirpath) and os.listdir(dirpath):
        latest_file = os.path.join(dirpath, os.listdir(dirpath)[-1])
    else:
        latest_file = os.path.join(dirpath, get_timestamp_str() + '.yaml')
    return load_yaml(latest_file, default)

def save_yaml(filepath, data):
    dirpath = os.path.dirname(os.path.abspath(filepath))
    if not os.path.exists(dirpath):
        os.makedirs(dirpath)
    with open(filepath, 'w') as f:
        yaml.dump(data, f, default_flow_style=False, width=1000**2)

def save_versioned_yaml(dirpath, data, ensure_ascii=True, sort_keys=True, indent=4):
    filepath = os.path.join(dirpath, get_timestamp_str() + '.yaml')
    save_yaml(filepath, data)
    return filepath

def split_path(filepath):
    dirpath, filename = os.path.split(filepath)
    file, ext = os.path.splitext(filename)
    part, name = parse_part(file)
    return name, ext, part, dirpath

def pretty_print_seconds(seconds):
    return str(timedelta(seconds=seconds))

def pretty_print_bytes(size_bytes):
    if size_bytes == 0:
        return "0B"
    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)
    return "%s%s" % (s, size_name[i])

def parse_date_str(s):
    p1, p2, p3 = [int(x) for x in re.split('\.|\-|\_|\s', s.strip().removeprefix('(').removesuffix(')')) if x]
    current_yy = date.today().year - 2000
    year = None
    month = None
    day = None

    if p1 > current_yy and p1 <= 31:
        day = p1
        month = p2
        year = p3 % 2000 + 2000
        return f'{year}-{str(month).zfill(2)}-{day}'
    elif p2 > 12:
        day = p2
        month = p1
        year = p3 % 2000 + 2000
        return f'{year}-{str(month).zfill(2)}-{day}'
    elif p3 > current_yy and p3 <= 31:
        day = p3
        month = p2
        year = p1 % 2000 + 2000
        return f'{year}-{str(month).zfill(2)}-{day}'
    elif p1 >= 2000:
        day = p3
        month = p2
        year = p1 % 2000 + 2000
        return f'{year}-{str(month).zfill(2)}-{day}'

    if p1 == p3:
        year = p1 % 2000 + 2000

    if p1 > 12:
        month = p2

        if p1 == p3:
            day = p3
            month = p2
            year = p1 % 2000 + 2000
            return f'{year}-{str(month).zfill(2)}-{day}'
        
    if p3 >= 2000:
        year = p3 % 2000 + 2000
        
        if p1 > 12:
            day = p1
            return f'{year}-{str(month).zfill(2)}-{day}'
            
        elif p2 > 12:
            day = p2
            month = p1
            return f'{year}-{str(month).zfill(2)}-{day}'

        elif p1 == p2:
            day = p1
            month = p2
            return f'{year}-{str(month).zfill(2)}-{day}'

    return None


import sqlite3
from . import sqlite_wrapper as sq

class Table(object):
    
    def __init__(self, conn: sqlite3.Connection, table_name):
        self._table_name = table_name
        self._conn = conn
        self._columns = sq.db_table_columns(self._conn, table_name)

    @property
    def table_name(self):
        return self._table_name

    @property
    def conn(self):
        return self._conn

    @conn.setter
    def conn(self, conn: sqlite3.Connection):
        self._conn = conn

    def execute(self, sql, vals, commit=True):
        return sq.db_execute(self.conn, sql, vals, commit)

    def select(self, colvalues=None, selectcols=['*']):
        return sq.db_select(self.conn, self._table_name, colvalues, selectcols)

    def selectone(self, colvalues=None, selectcols=['*']):
        return sq.db_selectone(self.conn, self._table_name, colvalues, selectcols)

    def row_update(self, colvalues, keys, wherevalues=None, check_empty=False, commit=True):
        return sq.db_update(self.conn, self._table_name, colvalues, keys, wherevalues, check_empty, commit)

    def row_insert(self, colvalues, commit=True):
        return sq.db_insert(self.conn, self._table_name, colvalues, commit)
        
    def row_upsert(self, colvalues, keys=None, wherevalues=None, check_empty=False, commit=True):
        return sq.db_upsert(self.conn, self._table_name, colvalues, keys, wherevalues, check_empty, commit)

class TableRow(object):
    def __init__(self, table_name):
        self._table_name = table_name

    def from_dict(self, row_dict):
        for col, val in row_dict.items():
            setattr(self, col, val)
        return self

    def from_sqliterow(self, row):
        for col in row.keys():
            setattr(self, col, row[col])
        return self


import sqlite3
from .common import dict_subset, array_param
    
def db_select(conn: sqlite3.Connection, table, colvalues=None, selectcols=['*']):
    vals = []
    where = ''
    if colvalues:
        cols, vals = zip(*colvalues.items())
        where = ' WHERE ' + ' AND '.join([col + ' = ?' for col in cols])
    select = ','.join([col for col in selectcols])
    sql = f"""SELECT {select} FROM {table}{where}"""
    # print(sql, vals)
    c = conn.execute(sql, vals)
    results = c.fetchall()
    if len(selectcols) == 1 and selectcols[0] != '*':
        return [x[0] for x in results]
    else:
        return results

def db_selectone(conn: sqlite3.Connection, table, colvalues=None, selectcols=['*']):
    results = db_select(conn, table, colvalues, selectcols)
    if results:
        return results[0]
    else:
        return None

def db_tables(conn: sqlite3.Connection):
    return db_select(conn, 'sqlite_master', {'type': 'table'}, ['name'])

def db_table_columns(conn: sqlite3.Connection, table):
    c = conn.execute(f"""PRAGMA table_info({table});""")
    return [x[1] for x in c.fetchall()]


def db_execute(conn: sqlite3.Connection, sql, vals, commit=True):
    # print('execute', sql, vals)
    c = conn.execute(sql, vals)
    if commit:
        conn.commit()
    return c

def db_update(conn: sqlite3.Connection, table, colvalues, keys, wherevalues=None, check_empty=False, commit=True):
    cols, vals = zip(*colvalues.items())
    where = ' AND '.join([col + ' = ?' for col in keys])
    if check_empty:
        where  += ' AND ' + ' AND '.join([f"({col} IS NULL OR {col} = '' OR {col} = 0)" for col in cols])
    setvals = ', '.join([col + ' = ?' for col in cols])
    sql = f"""UPDATE {table} SET {setvals} WHERE {where}"""
    if not wherevalues:
        wherevalues = colvalues
    vals += tuple([wherevalues[key] for key in keys])
    return db_execute(conn, sql, vals, commit)

def db_insert(conn: sqlite3.Connection, table, colvalues, commit=True):
    cols, vals = zip(*colvalues.items())
    sql = f"""INSERT INTO {table} ({', '.join(cols)}) VALUES ({array_param(vals)})"""
    return db_execute(conn, sql, vals, commit)
    
def db_upsert(conn: sqlite3.Connection, table, colvalues, keys=None, wherevalues=None, check_empty=False, commit=True):
    if keys:
        keyvals = dict_subset(colvalues, keys)
        rows = db_select(conn, table, keyvals)
        if rows:
            return db_update(conn, table, colvalues, keys, wherevalues, check_empty, commit)
    return db_insert(conn, table, colvalues, commit)



from . import sqlite_wrapper as sq
from .database import Database
from .stash_tables import *

class StashDatabaseBase(Database):

	SCHEMA_VERSION = 28

	def __init__(self, db_path):
		super().__init__(db_path)
		self.schema_migrations = SchemaMigrations(self.conn)
		self.tags = Tags(self.conn)
		self.sqlite_sequence = SqliteSequence(self.conn)
		self.studios = Studios(self.conn)
		self.performers = Performers(self.conn)
		self.movies = Movies(self.conn)
		self.scraped_items = ScrapedItems(self.conn)
		self.performers_image = PerformersImage(self.conn)
		self.studios_image = StudiosImage(self.conn)
		self.movies_images = MoviesImages(self.conn)
		self.tags_image = TagsImage(self.conn)
		self.scenes = Scenes(self.conn)
		self.performers_scenes = PerformersScenes(self.conn)
		self.scene_markers = SceneMarkers(self.conn)
		self.scene_markers_tags = SceneMarkersTags(self.conn)
		self.scenes_tags = ScenesTags(self.conn)
		self.movies_scenes = MoviesScenes(self.conn)
		self.scenes_cover = ScenesCover(self.conn)
		self.images = Images(self.conn)
		self.performers_images = PerformersImages(self.conn)
		self.images_tags = ImagesTags(self.conn)
		self.scene_stash_ids = SceneStashIds(self.conn)
		self.performer_stash_ids = PerformerStashIds(self.conn)
		self.studio_stash_ids = StudioStashIds(self.conn)
		self.galleries = Galleries(self.conn)
		self.scenes_galleries = ScenesGalleries(self.conn)
		self.galleries_images = GalleriesImages(self.conn)
		self.performers_galleries = PerformersGalleries(self.conn)
		self.galleries_tags = GalleriesTags(self.conn)
		self.performers_tags = PerformersTags(self.conn)
		self.tag_aliases = TagAliases(self.conn)
		self.saved_filters = SavedFilters(self.conn)
		self.tags_relations = TagsRelations(self.conn)
		self.studio_aliases = StudioAliases(self.conn)



from enum import IntEnum
from . import log

class LogLevel(IntEnum):
	TRACE = 1
	DEBUG = 2
	INFO = 3
	WARN = 4
	ERROR = 5

class Logger:

	_prefixes = {
		b't': '[TRACE]',
		b'd': '[DEBUG]',
		b'i': '[INFO]',
		b'w': '[WARN]',
		b'e': '[ERROR]',
	}

	def __init__(self, log_level=LogLevel.TRACE, plugin=True):
		self.log_level = log_level
		self.plugin = plugin

	def _log(self, level_char, s):
		if self.plugin:
			log.log(level_char, s)
		else:
			print(self._prefixes[level_char], s)

	def LogTrace(self, s):
		if self.log_level <= LogLevel.TRACE:
			self._log(b't', s)


	def LogDebug(self, s):
		if self.log_level <= LogLevel.DEBUG:
			self._log(b'd', s)


	def LogInfo(self, s):
		if self.log_level <= LogLevel.INFO:
			self._log(b'i', s)


	def LogWarning(self, s):
		if self.log_level <= LogLevel.WARN:
			self._log(b'w', s)


	def LogError(self, s):
		if self.log_level <= LogLevel.ERROR:
			self._log(b'e', s)


	def LogProgress(self, p):
		if self.plugin:
			progress = min(max(0, p), 1)
			self._log(b'p', str(progress))

logger = Logger()


import os
import re
import sqlite3
from . import sqlite_wrapper as sq
from .common import alphanum_chars
from .logger import logger as log

def regexp(expr, item):
    reg = re.compile(expr)
    return reg.search(item) is not None

def studio_matcher(expr, item):
    return alphanum_chars(expr).lower() == alphanum_chars(item).lower()

class Database(object):

    def __init__(self, db_path):
        try:
            if not os.path.isfile(db_path):
                raise Exception(f'database file missing {db_path}')
            self._conn = sqlite3.connect(db_path)
            self._conn.row_factory = sqlite3.Row
            self._conn.execute("PRAGMA foreign_keys = 1")
            self._conn.create_function("REGEXP", 2, regexp)
            self._conn.create_function("STUDIOMATCHER", 2, studio_matcher)
        except sqlite3.Error as e:
            raise Exception("error connecting to database!")
        self._tables = sq.db_tables(self._conn)

    @property
    def conn(self):
        return self._conn

    @property
    def tables(self):
        return self._tables

    def close(self):
        if self.conn:
            self.conn.commit()
            self.conn.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def commit(self):
        self.conn.commit()

    def execute(self, sql, vals=[], commit=True):
        return sq.db_execute(self.conn, sql, vals, commit)

    def fetchall(self, sql, vals=[]):
        c = sq.db_execute(self.conn, sql, vals, commit=False)
        return c.fetchall()

    def fetchone(self, sql, vals=[]):
        results = self.fetchall(sql, vals)
        if results:
            return results[0]
        else:
            return None

    def table_columns(self, table_name):
        return sq.db_table_columns(self.conn, table_name)


import os
import re
from .common import get_checksum, get_timestamp, image_to_base64, parse_part, optional_nonalphanum_regex
from .logger import logger as log
from .stash_database_base import StashDatabaseBase
from .stash_tables import *
from .stash_models import *

class StashDatabase(StashDatabaseBase):

    def __init__(self, db_path):
        super().__init__(db_path)
        row = self.schema_migrations.selectone()
        if not row:
            raise Exception(f'schema version not found')
        schema_migration = SchemaMigrationsRow().from_sqliterow(row)
        if schema_migration.version != StashDatabase.SCHEMA_VERSION:
            raise Exception(f'schema version mismatch. schema version is {schema_migration.version}, expected {StashDatabase.SCHEMA_VERSION}')
        elif schema_migration.dirty:
            raise Exception(f'schema migration not completed')

    def performers_from_scene_id(self, scene_id):
        rows = self.fetchall("""SELECT c.* FROM scenes a JOIN performers_scenes b ON a.id = b.scene_id JOIN performers c ON c.id = b.performer_id WHERE a.id = ?""", (scene_id, ))
        return [PerformersRow().from_sqliterow(row) for row in rows]

    def query_performer_url(self, url):
        result = self.fetchone(f"""SELECT * FROM performers WHERE url LIKE ?""", (url, ))
        if result:
            return PerformersRow().from_sqliterow(result)
        return None

    def query_performer_name(self, name):
        result = self.fetchone(f"""SELECT * FROM performers WHERE name LIKE ?""", (name, ))
        if result:
            return PerformersRow().from_sqliterow(result)
        return None

    def query_performer_name_regex(self, name):
        name_regex = optional_nonalphanum_regex(name)
        rows = self.fetchall(f"""SELECT * FROM performers WHERE name REGEXP ?""", (name_regex, ))
        return [PerformersRow().from_sqliterow(row) for row in rows]

    def query_studio_name(self, name):
        result = self.fetchone(f"""SELECT * FROM studios WHERE name LIKE ?""", (name, ))
        if result:
            return StudiosRow().from_sqliterow(result)
        return None

    def query_studio_name_regex(self, name):
        rows = self.fetchall(f"""SELECT * FROM studios WHERE STUDIOMATCHER(name, ?)""", (name, ))
        return [StudiosRow().from_sqliterow(row) for row in rows]

    def query_performer_alias(self, name):
        rows = self.fetchall(f"""SELECT * FROM performers WHERE aliases LIKE ?""", ('%' + name + '%', ))
        performers = [PerformersRow().from_sqliterow(row) for row in rows]
        results = []
        name_key = re.sub(r'[^a-zA-Z0-9\s]+', '', name.lower()).strip()
        for performer in performers:
            aliases = [x.split('(')[0].lower().strip() for x in performer.aliases.split(',')]
            aliases = [re.sub(r'[^a-zA-Z0-9\s]+', '', x).strip() for x in aliases]
            for alias in aliases:
                if alias == name_key:
                    results.append(performer)
                    match_found = True
                    break
        return results

    def query_studio_name(self, name):
        result = self.fetchone(f"""SELECT * FROM studios WHERE REPLACE(REPLACE(name, ' ', ''), '-', '') LIKE ?""", (name.replace(' ', '').replace('-', ''), ))
        if result:
            return StudiosRow().from_sqliterow(result)
        return None

    def insert_performer(self, performer: PerformersRow, commit=True):
        performer.checksum = get_checksum(performer.name)
        performer.created_at = get_timestamp()
        performer.updated_at = get_timestamp()
        c = self.performers.insert_model(performer, commit)
        if c.rowcount:
            return self.performers.selectone_name(performer.name)
        return None

    def create_performer_from_url(self, name, url):
        performer = self.performers.selectone_url(url)
        if performer:
            return performer

        results = self.query_performer_name(name)
        if results:
            return None

        performer = PerformersRow()
        performer.name = name
        performer.gender = 'FEMALE'
        performer.url = url
        performer.favorite = 0
        performer.details = None

        performer = self.insert_performer(performer)
        if performer:
            return performer
        return None


    def insert_performer_image(self, performer: PerformersRow, image_bytes: bytes, commit=True):
        performer_image = self.performers_image.selectone_performer_id(performer.id)
        if not performer_image:
            performer_image = PerformersImageRow()
            performer_image.performer_id = performer.id
            performer_image.image = image_bytes
            c = self.performers_image.insert_model(performer_image, commit)
            if c.rowcount:
                return self.performers_image.selectone_performer_id(performer.id)
            return None
        else:
            return performer_image

    def add_performers_to_scene(self, scene: ScenesRow, performers: list[PerformersRow], commit=True):
        existing_performer_ids = [performer_scene.performer_id for performer_scene in self.performers_scenes.select_scene_id(scene.id)]
        for performer in performers:
            if performer.id in existing_performer_ids:
                continue
            performers_scenes = PerformersScenesRow()
            performers_scenes.performer_id = performer.id
            performers_scenes.scene_id = scene.id
            try:
                self.performers_scenes.insert_model(performers_scenes, commit=False)
            except Exception as e:
                log.LogError(str(e))
                pass
        if commit:
            self.commit()

    def add_tag_to_scene(self, scene: ScenesRow, tag: TagsRow, commit=True):
        scenes_tags = ScenesTagsRow()
        scenes_tags.scene_id = scene.id
        scenes_tags.tag_id = tag.id
        existing_scene_tag = self.scenes_tags.selectone({'scene_id': scene.id, 'tag_id': tag.id})
        if not existing_scene_tag:
            self.scenes_tags.insert_model(scenes_tags, commit)

    def add_tag_name_to_scene(self, scene: ScenesRow, tag_name, commit=True):
        tag = self.tags.selectone_name(tag_name)
        if not tag:
            return False
        scenes_tags = ScenesTagsRow()
        scenes_tags.scene_id = scene.id
        scenes_tags.tag_id = tag.id
        existing_scene_tag = self.scenes_tags.selectone({'scene_id': scene.id, 'tag_id': tag.id})
        if not existing_scene_tag:
            self.scenes_tags.insert_model(scenes_tags, commit)
        return True

    def add_scenes_to_movie(self, movie: MoviesRow, scenes: list[ScenesRow], scene_index=[], commit=True):
        for i in range(0, len(scenes)):
            scene = scenes[i]
            movies_scenes = MoviesScenesRow()
            movies_scenes.movie_id = movie.id
            movies_scenes.scene_id = scene.id
            if scene_index:
                movies_scenes.scene_index = scene_index[i]
            try:
                self.movies_scenes.insert_model(movies_scenes, commit=False)
            except Exception as e:
                log.LogError(str(e))
                pass
        if commit:
            self.commit()

    def create_movie(self, name, url, front_path, back_path):
        # get existing movie by name
        movie = self.movies.selectone_name(name)

        # insert new movie if does not exist
        if not movie:
            movie = MoviesRow
            movie.name = name
            movie.checksum = get_checksum(name)
            movie.url = url
            movie.created_at = get_timestamp()
            movie.updated_at = get_timestamp()
            self.movies.insert_model(movie)
            movie = self.movies.selectone_name(name)

        # get front/back image contents
        front_image = None
        if front_image:
            front_image = image_to_base64(open(front_path, 'rb').read())
        back_image = None
        if back_path:
            back_image = image_to_base64(open(back_path, 'rb').read())

        # insert or update front/back images
        movies_images = self.movies_images.selectone_movie_id(movie.id)
        if not movies_images:
            movies_images = MoviesImagesRow()
            movies_images.movie_id = movie.id
            movies_images.front_image = front_image
            movies_images.back_image = back_image
            self.movies_images.insert_model(movies_images)
        else:
            self.movies_images.update_front_image_by_movie_id(movie.id, front_image, commit=False)
            self.movies_images.update_front_image_by_movie_id(movie.id, back_image, commit=False)
            self.commit()

        return movie

    def create_movie_from_folder(self, dirpath, url, front_path, back_path, front_only=False):
        name = os.path.basename(dirpath)
        name = re.sub(' \[.*?\]', '', name)
        name = re.sub(' \(\d{4}\)', '', name)

        # get front/back image paths
        if not front_path:
            front_path = os.path.join(dirpath, 'front.jpg')
        if not back_path and not front_only:
            back_path = os.path.join(dirpath, 'back.jpg')

        movie = self.create_movie(name, url, front_path, back_path)

        # add scene from files in folder to movie
        filepaths = [os.path.join(dirpath, file) for file in os.listdir(dirpath)]
        scenes = [self.scenes.selectone_path(filepath) for filepath in filepaths]
        scenes = [scene for scene in scenes if scene]
        self.add_scenes_to_movie(movie, scenes)

    def create_movie_from_file(self, filepath, url, front_path, back_path):
        filename = os.path.basename(filepath)
        part, name = parse_part(os.path.splitext(filename)[0])

        movie = self.create_movie(name, url, front_path, back_path)

        # add scene from file to movie
        scene = self.scenes.selectone_path(filepath)
        if scene:
            self.add_scenes_to_movie(movie, [scene], scene_index=[part])

    def add_performers_to_gallery(self, gallery: GalleriesRow, performers: list[PerformersRow], commit=True):
        existing_performer_ids = [performer_gallery.performer_id for performer_gallery in self.performers_galleries.select_gallery_id(gallery.id)]
        for performer in performers:
            if performer.id in existing_performer_ids:
                continue
            performers_galleries = PerformersGalleriesRow()
            performers_galleries.performer_id = performer.id
            performers_galleries.gallery_id = gallery.id
            try:
                self.performers_galleries.insert_model(performers_galleries, commit=False)
            except Exception as e:
                log.LogError(str(e))
                pass
        if commit:
            self.commit()



from .table import TableRow

class SchemaMigrationsRow(TableRow):
	def __init__(self):
		super().__init__('schema_migrations')
		self._version = None
		self._dirty = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def version(self):
		return self._version

	@version.setter
	def version(self, version):
		self._version = version

	@property
	def dirty(self):
		return self._dirty

	@dirty.setter
	def dirty(self, dirty):
		self._dirty = dirty

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.version, self.dirty]
		else:
			return [self.version, self.dirty]

class TagsRow(TableRow):
	def __init__(self):
		super().__init__('tags')
		self._id = None
		self._name = None
		self._created_at = None
		self._updated_at = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def id(self):
		return self._id

	@id.setter
	def id(self, id):
		self._id = id

	@property
	def name(self):
		return self._name

	@name.setter
	def name(self, name):
		self._name = name

	@property
	def created_at(self):
		return self._created_at

	@created_at.setter
	def created_at(self, created_at):
		self._created_at = created_at

	@property
	def updated_at(self):
		return self._updated_at

	@updated_at.setter
	def updated_at(self, updated_at):
		self._updated_at = updated_at

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.id, self.name, self.created_at, self.updated_at]
		else:
			return [self.name, self.created_at, self.updated_at]

class SqliteSequenceRow(TableRow):
	def __init__(self):
		super().__init__('sqlite_sequence')
		self._name = None
		self._seq = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def name(self):
		return self._name

	@name.setter
	def name(self, name):
		self._name = name

	@property
	def seq(self):
		return self._seq

	@seq.setter
	def seq(self, seq):
		self._seq = seq

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.name, self.seq]
		else:
			return [self.name, self.seq]

class StudiosRow(TableRow):
	def __init__(self):
		super().__init__('studios')
		self._id = None
		self._checksum = None
		self._name = None
		self._url = None
		self._parent_id = None
		self._created_at = None
		self._updated_at = None
		self._details = None
		self._rating = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def id(self):
		return self._id

	@id.setter
	def id(self, id):
		self._id = id

	@property
	def checksum(self):
		return self._checksum

	@checksum.setter
	def checksum(self, checksum):
		self._checksum = checksum

	@property
	def name(self):
		return self._name

	@name.setter
	def name(self, name):
		self._name = name

	@property
	def url(self):
		return self._url

	@url.setter
	def url(self, url):
		self._url = url

	@property
	def parent_id(self):
		return self._parent_id

	@parent_id.setter
	def parent_id(self, parent_id):
		self._parent_id = parent_id

	@property
	def created_at(self):
		return self._created_at

	@created_at.setter
	def created_at(self, created_at):
		self._created_at = created_at

	@property
	def updated_at(self):
		return self._updated_at

	@updated_at.setter
	def updated_at(self, updated_at):
		self._updated_at = updated_at

	@property
	def details(self):
		return self._details

	@details.setter
	def details(self, details):
		self._details = details

	@property
	def rating(self):
		return self._rating

	@rating.setter
	def rating(self, rating):
		self._rating = rating

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.id, self.checksum, self.name, self.url, self.parent_id, self.created_at, self.updated_at, self.details, self.rating]
		else:
			return [self.checksum, self.name, self.url, self.parent_id, self.created_at, self.updated_at, self.details, self.rating]

class PerformersRow(TableRow):
	def __init__(self):
		super().__init__('performers')
		self._id = None
		self._checksum = None
		self._name = None
		self._gender = None
		self._url = None
		self._twitter = None
		self._instagram = None
		self._birthdate = None
		self._ethnicity = None
		self._country = None
		self._eye_color = None
		self._height = None
		self._measurements = None
		self._fake_tits = None
		self._career_length = None
		self._tattoos = None
		self._piercings = None
		self._aliases = None
		self._favorite = None
		self._created_at = None
		self._updated_at = None
		self._details = None
		self._death_date = None
		self._hair_color = None
		self._weight = None
		self._rating = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def id(self):
		return self._id

	@id.setter
	def id(self, id):
		self._id = id

	@property
	def checksum(self):
		return self._checksum

	@checksum.setter
	def checksum(self, checksum):
		self._checksum = checksum

	@property
	def name(self):
		return self._name

	@name.setter
	def name(self, name):
		self._name = name

	@property
	def gender(self):
		return self._gender

	@gender.setter
	def gender(self, gender):
		self._gender = gender

	@property
	def url(self):
		return self._url

	@url.setter
	def url(self, url):
		self._url = url

	@property
	def twitter(self):
		return self._twitter

	@twitter.setter
	def twitter(self, twitter):
		self._twitter = twitter

	@property
	def instagram(self):
		return self._instagram

	@instagram.setter
	def instagram(self, instagram):
		self._instagram = instagram

	@property
	def birthdate(self):
		return self._birthdate

	@birthdate.setter
	def birthdate(self, birthdate):
		self._birthdate = birthdate

	@property
	def ethnicity(self):
		return self._ethnicity

	@ethnicity.setter
	def ethnicity(self, ethnicity):
		self._ethnicity = ethnicity

	@property
	def country(self):
		return self._country

	@country.setter
	def country(self, country):
		self._country = country

	@property
	def eye_color(self):
		return self._eye_color

	@eye_color.setter
	def eye_color(self, eye_color):
		self._eye_color = eye_color

	@property
	def height(self):
		return self._height

	@height.setter
	def height(self, height):
		self._height = height

	@property
	def measurements(self):
		return self._measurements

	@measurements.setter
	def measurements(self, measurements):
		self._measurements = measurements

	@property
	def fake_tits(self):
		return self._fake_tits

	@fake_tits.setter
	def fake_tits(self, fake_tits):
		self._fake_tits = fake_tits

	@property
	def career_length(self):
		return self._career_length

	@career_length.setter
	def career_length(self, career_length):
		self._career_length = career_length

	@property
	def tattoos(self):
		return self._tattoos

	@tattoos.setter
	def tattoos(self, tattoos):
		self._tattoos = tattoos

	@property
	def piercings(self):
		return self._piercings

	@piercings.setter
	def piercings(self, piercings):
		self._piercings = piercings

	@property
	def aliases(self):
		return self._aliases

	@aliases.setter
	def aliases(self, aliases):
		self._aliases = aliases

	@property
	def favorite(self):
		return self._favorite

	@favorite.setter
	def favorite(self, favorite):
		self._favorite = favorite

	@property
	def created_at(self):
		return self._created_at

	@created_at.setter
	def created_at(self, created_at):
		self._created_at = created_at

	@property
	def updated_at(self):
		return self._updated_at

	@updated_at.setter
	def updated_at(self, updated_at):
		self._updated_at = updated_at

	@property
	def details(self):
		return self._details

	@details.setter
	def details(self, details):
		self._details = details

	@property
	def death_date(self):
		return self._death_date

	@death_date.setter
	def death_date(self, death_date):
		self._death_date = death_date

	@property
	def hair_color(self):
		return self._hair_color

	@hair_color.setter
	def hair_color(self, hair_color):
		self._hair_color = hair_color

	@property
	def weight(self):
		return self._weight

	@weight.setter
	def weight(self, weight):
		self._weight = weight

	@property
	def rating(self):
		return self._rating

	@rating.setter
	def rating(self, rating):
		self._rating = rating

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.id, self.checksum, self.name, self.gender, self.url, self.twitter, self.instagram, self.birthdate, self.ethnicity, self.country, self.eye_color, self.height, self.measurements, self.fake_tits, self.career_length, self.tattoos, self.piercings, self.aliases, self.favorite, self.created_at, self.updated_at, self.details, self.death_date, self.hair_color, self.weight, self.rating]
		else:
			return [self.checksum, self.name, self.gender, self.url, self.twitter, self.instagram, self.birthdate, self.ethnicity, self.country, self.eye_color, self.height, self.measurements, self.fake_tits, self.career_length, self.tattoos, self.piercings, self.aliases, self.favorite, self.created_at, self.updated_at, self.details, self.death_date, self.hair_color, self.weight, self.rating]

class MoviesRow(TableRow):
	def __init__(self):
		super().__init__('movies')
		self._id = None
		self._name = None
		self._aliases = None
		self._duration = None
		self._date = None
		self._rating = None
		self._studio_id = None
		self._director = None
		self._synopsis = None
		self._checksum = None
		self._url = None
		self._created_at = None
		self._updated_at = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def id(self):
		return self._id

	@id.setter
	def id(self, id):
		self._id = id

	@property
	def name(self):
		return self._name

	@name.setter
	def name(self, name):
		self._name = name

	@property
	def aliases(self):
		return self._aliases

	@aliases.setter
	def aliases(self, aliases):
		self._aliases = aliases

	@property
	def duration(self):
		return self._duration

	@duration.setter
	def duration(self, duration):
		self._duration = duration

	@property
	def date(self):
		return self._date

	@date.setter
	def date(self, date):
		self._date = date

	@property
	def rating(self):
		return self._rating

	@rating.setter
	def rating(self, rating):
		self._rating = rating

	@property
	def studio_id(self):
		return self._studio_id

	@studio_id.setter
	def studio_id(self, studio_id):
		self._studio_id = studio_id

	@property
	def director(self):
		return self._director

	@director.setter
	def director(self, director):
		self._director = director

	@property
	def synopsis(self):
		return self._synopsis

	@synopsis.setter
	def synopsis(self, synopsis):
		self._synopsis = synopsis

	@property
	def checksum(self):
		return self._checksum

	@checksum.setter
	def checksum(self, checksum):
		self._checksum = checksum

	@property
	def url(self):
		return self._url

	@url.setter
	def url(self, url):
		self._url = url

	@property
	def created_at(self):
		return self._created_at

	@created_at.setter
	def created_at(self, created_at):
		self._created_at = created_at

	@property
	def updated_at(self):
		return self._updated_at

	@updated_at.setter
	def updated_at(self, updated_at):
		self._updated_at = updated_at

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.id, self.name, self.aliases, self.duration, self.date, self.rating, self.studio_id, self.director, self.synopsis, self.checksum, self.url, self.created_at, self.updated_at]
		else:
			return [self.name, self.aliases, self.duration, self.date, self.rating, self.studio_id, self.director, self.synopsis, self.checksum, self.url, self.created_at, self.updated_at]

class ScrapedItemsRow(TableRow):
	def __init__(self):
		super().__init__('scraped_items')
		self._id = None
		self._title = None
		self._description = None
		self._url = None
		self._date = None
		self._rating = None
		self._tags = None
		self._models = None
		self._episode = None
		self._gallery_filename = None
		self._gallery_url = None
		self._video_filename = None
		self._video_url = None
		self._studio_id = None
		self._created_at = None
		self._updated_at = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def id(self):
		return self._id

	@id.setter
	def id(self, id):
		self._id = id

	@property
	def title(self):
		return self._title

	@title.setter
	def title(self, title):
		self._title = title

	@property
	def description(self):
		return self._description

	@description.setter
	def description(self, description):
		self._description = description

	@property
	def url(self):
		return self._url

	@url.setter
	def url(self, url):
		self._url = url

	@property
	def date(self):
		return self._date

	@date.setter
	def date(self, date):
		self._date = date

	@property
	def rating(self):
		return self._rating

	@rating.setter
	def rating(self, rating):
		self._rating = rating

	@property
	def tags(self):
		return self._tags

	@tags.setter
	def tags(self, tags):
		self._tags = tags

	@property
	def models(self):
		return self._models

	@models.setter
	def models(self, models):
		self._models = models

	@property
	def episode(self):
		return self._episode

	@episode.setter
	def episode(self, episode):
		self._episode = episode

	@property
	def gallery_filename(self):
		return self._gallery_filename

	@gallery_filename.setter
	def gallery_filename(self, gallery_filename):
		self._gallery_filename = gallery_filename

	@property
	def gallery_url(self):
		return self._gallery_url

	@gallery_url.setter
	def gallery_url(self, gallery_url):
		self._gallery_url = gallery_url

	@property
	def video_filename(self):
		return self._video_filename

	@video_filename.setter
	def video_filename(self, video_filename):
		self._video_filename = video_filename

	@property
	def video_url(self):
		return self._video_url

	@video_url.setter
	def video_url(self, video_url):
		self._video_url = video_url

	@property
	def studio_id(self):
		return self._studio_id

	@studio_id.setter
	def studio_id(self, studio_id):
		self._studio_id = studio_id

	@property
	def created_at(self):
		return self._created_at

	@created_at.setter
	def created_at(self, created_at):
		self._created_at = created_at

	@property
	def updated_at(self):
		return self._updated_at

	@updated_at.setter
	def updated_at(self, updated_at):
		self._updated_at = updated_at

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.id, self.title, self.description, self.url, self.date, self.rating, self.tags, self.models, self.episode, self.gallery_filename, self.gallery_url, self.video_filename, self.video_url, self.studio_id, self.created_at, self.updated_at]
		else:
			return [self.title, self.description, self.url, self.date, self.rating, self.tags, self.models, self.episode, self.gallery_filename, self.gallery_url, self.video_filename, self.video_url, self.studio_id, self.created_at, self.updated_at]

class PerformersImageRow(TableRow):
	def __init__(self):
		super().__init__('performers_image')
		self._performer_id = None
		self._image = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def performer_id(self):
		return self._performer_id

	@performer_id.setter
	def performer_id(self, performer_id):
		self._performer_id = performer_id

	@property
	def image(self):
		return self._image

	@image.setter
	def image(self, image):
		self._image = image

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.performer_id, self.image]
		else:
			return [self.performer_id, self.image]

class StudiosImageRow(TableRow):
	def __init__(self):
		super().__init__('studios_image')
		self._studio_id = None
		self._image = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def studio_id(self):
		return self._studio_id

	@studio_id.setter
	def studio_id(self, studio_id):
		self._studio_id = studio_id

	@property
	def image(self):
		return self._image

	@image.setter
	def image(self, image):
		self._image = image

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.studio_id, self.image]
		else:
			return [self.studio_id, self.image]

class MoviesImagesRow(TableRow):
	def __init__(self):
		super().__init__('movies_images')
		self._movie_id = None
		self._front_image = None
		self._back_image = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def movie_id(self):
		return self._movie_id

	@movie_id.setter
	def movie_id(self, movie_id):
		self._movie_id = movie_id

	@property
	def front_image(self):
		return self._front_image

	@front_image.setter
	def front_image(self, front_image):
		self._front_image = front_image

	@property
	def back_image(self):
		return self._back_image

	@back_image.setter
	def back_image(self, back_image):
		self._back_image = back_image

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.movie_id, self.front_image, self.back_image]
		else:
			return [self.movie_id, self.front_image, self.back_image]

class TagsImageRow(TableRow):
	def __init__(self):
		super().__init__('tags_image')
		self._tag_id = None
		self._image = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def tag_id(self):
		return self._tag_id

	@tag_id.setter
	def tag_id(self, tag_id):
		self._tag_id = tag_id

	@property
	def image(self):
		return self._image

	@image.setter
	def image(self, image):
		self._image = image

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.tag_id, self.image]
		else:
			return [self.tag_id, self.image]

class ScenesRow(TableRow):
	def __init__(self):
		super().__init__('scenes')
		self._id = None
		self._path = None
		self._checksum = None
		self._oshash = None
		self._title = None
		self._details = None
		self._url = None
		self._date = None
		self._rating = None
		self._size = None
		self._duration = None
		self._video_codec = None
		self._audio_codec = None
		self._width = None
		self._height = None
		self._framerate = None
		self._bitrate = None
		self._studio_id = None
		self._o_counter = None
		self._format = None
		self._created_at = None
		self._updated_at = None
		self._file_mod_time = None
		self._organized = None
		self._phash = None
		self._interactive = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def id(self):
		return self._id

	@id.setter
	def id(self, id):
		self._id = id

	@property
	def path(self):
		return self._path

	@path.setter
	def path(self, path):
		self._path = path

	@property
	def checksum(self):
		return self._checksum

	@checksum.setter
	def checksum(self, checksum):
		self._checksum = checksum

	@property
	def oshash(self):
		return self._oshash

	@oshash.setter
	def oshash(self, oshash):
		self._oshash = oshash

	@property
	def title(self):
		return self._title

	@title.setter
	def title(self, title):
		self._title = title

	@property
	def details(self):
		return self._details

	@details.setter
	def details(self, details):
		self._details = details

	@property
	def url(self):
		return self._url

	@url.setter
	def url(self, url):
		self._url = url

	@property
	def date(self):
		return self._date

	@date.setter
	def date(self, date):
		self._date = date

	@property
	def rating(self):
		return self._rating

	@rating.setter
	def rating(self, rating):
		self._rating = rating

	@property
	def size(self):
		return self._size

	@size.setter
	def size(self, size):
		self._size = size

	@property
	def duration(self):
		return self._duration

	@duration.setter
	def duration(self, duration):
		self._duration = duration

	@property
	def video_codec(self):
		return self._video_codec

	@video_codec.setter
	def video_codec(self, video_codec):
		self._video_codec = video_codec

	@property
	def audio_codec(self):
		return self._audio_codec

	@audio_codec.setter
	def audio_codec(self, audio_codec):
		self._audio_codec = audio_codec

	@property
	def width(self):
		return self._width

	@width.setter
	def width(self, width):
		self._width = width

	@property
	def height(self):
		return self._height

	@height.setter
	def height(self, height):
		self._height = height

	@property
	def framerate(self):
		return self._framerate

	@framerate.setter
	def framerate(self, framerate):
		self._framerate = framerate

	@property
	def bitrate(self):
		return self._bitrate

	@bitrate.setter
	def bitrate(self, bitrate):
		self._bitrate = bitrate

	@property
	def studio_id(self):
		return self._studio_id

	@studio_id.setter
	def studio_id(self, studio_id):
		self._studio_id = studio_id

	@property
	def o_counter(self):
		return self._o_counter

	@o_counter.setter
	def o_counter(self, o_counter):
		self._o_counter = o_counter

	@property
	def format(self):
		return self._format

	@format.setter
	def format(self, format):
		self._format = format

	@property
	def created_at(self):
		return self._created_at

	@created_at.setter
	def created_at(self, created_at):
		self._created_at = created_at

	@property
	def updated_at(self):
		return self._updated_at

	@updated_at.setter
	def updated_at(self, updated_at):
		self._updated_at = updated_at

	@property
	def file_mod_time(self):
		return self._file_mod_time

	@file_mod_time.setter
	def file_mod_time(self, file_mod_time):
		self._file_mod_time = file_mod_time

	@property
	def organized(self):
		return self._organized

	@organized.setter
	def organized(self, organized):
		self._organized = organized

	@property
	def phash(self):
		return self._phash

	@phash.setter
	def phash(self, phash):
		self._phash = phash

	@property
	def interactive(self):
		return self._interactive

	@interactive.setter
	def interactive(self, interactive):
		self._interactive = interactive

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.id, self.path, self.checksum, self.oshash, self.title, self.details, self.url, self.date, self.rating, self.size, self.duration, self.video_codec, self.audio_codec, self.width, self.height, self.framerate, self.bitrate, self.studio_id, self.o_counter, self.format, self.created_at, self.updated_at, self.file_mod_time, self.organized, self.phash, self.interactive]
		else:
			return [self.path, self.checksum, self.oshash, self.title, self.details, self.url, self.date, self.rating, self.size, self.duration, self.video_codec, self.audio_codec, self.width, self.height, self.framerate, self.bitrate, self.studio_id, self.o_counter, self.format, self.created_at, self.updated_at, self.file_mod_time, self.organized, self.phash, self.interactive]

class PerformersScenesRow(TableRow):
	def __init__(self):
		super().__init__('performers_scenes')
		self._performer_id = None
		self._scene_id = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def performer_id(self):
		return self._performer_id

	@performer_id.setter
	def performer_id(self, performer_id):
		self._performer_id = performer_id

	@property
	def scene_id(self):
		return self._scene_id

	@scene_id.setter
	def scene_id(self, scene_id):
		self._scene_id = scene_id

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.performer_id, self.scene_id]
		else:
			return [self.performer_id, self.scene_id]

class SceneMarkersRow(TableRow):
	def __init__(self):
		super().__init__('scene_markers')
		self._id = None
		self._title = None
		self._seconds = None
		self._primary_tag_id = None
		self._scene_id = None
		self._created_at = None
		self._updated_at = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def id(self):
		return self._id

	@id.setter
	def id(self, id):
		self._id = id

	@property
	def title(self):
		return self._title

	@title.setter
	def title(self, title):
		self._title = title

	@property
	def seconds(self):
		return self._seconds

	@seconds.setter
	def seconds(self, seconds):
		self._seconds = seconds

	@property
	def primary_tag_id(self):
		return self._primary_tag_id

	@primary_tag_id.setter
	def primary_tag_id(self, primary_tag_id):
		self._primary_tag_id = primary_tag_id

	@property
	def scene_id(self):
		return self._scene_id

	@scene_id.setter
	def scene_id(self, scene_id):
		self._scene_id = scene_id

	@property
	def created_at(self):
		return self._created_at

	@created_at.setter
	def created_at(self, created_at):
		self._created_at = created_at

	@property
	def updated_at(self):
		return self._updated_at

	@updated_at.setter
	def updated_at(self, updated_at):
		self._updated_at = updated_at

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.id, self.title, self.seconds, self.primary_tag_id, self.scene_id, self.created_at, self.updated_at]
		else:
			return [self.title, self.seconds, self.primary_tag_id, self.scene_id, self.created_at, self.updated_at]

class SceneMarkersTagsRow(TableRow):
	def __init__(self):
		super().__init__('scene_markers_tags')
		self._scene_marker_id = None
		self._tag_id = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def scene_marker_id(self):
		return self._scene_marker_id

	@scene_marker_id.setter
	def scene_marker_id(self, scene_marker_id):
		self._scene_marker_id = scene_marker_id

	@property
	def tag_id(self):
		return self._tag_id

	@tag_id.setter
	def tag_id(self, tag_id):
		self._tag_id = tag_id

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.scene_marker_id, self.tag_id]
		else:
			return [self.scene_marker_id, self.tag_id]

class ScenesTagsRow(TableRow):
	def __init__(self):
		super().__init__('scenes_tags')
		self._scene_id = None
		self._tag_id = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def scene_id(self):
		return self._scene_id

	@scene_id.setter
	def scene_id(self, scene_id):
		self._scene_id = scene_id

	@property
	def tag_id(self):
		return self._tag_id

	@tag_id.setter
	def tag_id(self, tag_id):
		self._tag_id = tag_id

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.scene_id, self.tag_id]
		else:
			return [self.scene_id, self.tag_id]

class MoviesScenesRow(TableRow):
	def __init__(self):
		super().__init__('movies_scenes')
		self._movie_id = None
		self._scene_id = None
		self._scene_index = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def movie_id(self):
		return self._movie_id

	@movie_id.setter
	def movie_id(self, movie_id):
		self._movie_id = movie_id

	@property
	def scene_id(self):
		return self._scene_id

	@scene_id.setter
	def scene_id(self, scene_id):
		self._scene_id = scene_id

	@property
	def scene_index(self):
		return self._scene_index

	@scene_index.setter
	def scene_index(self, scene_index):
		self._scene_index = scene_index

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.movie_id, self.scene_id, self.scene_index]
		else:
			return [self.movie_id, self.scene_id, self.scene_index]

class ScenesCoverRow(TableRow):
	def __init__(self):
		super().__init__('scenes_cover')
		self._scene_id = None
		self._cover = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def scene_id(self):
		return self._scene_id

	@scene_id.setter
	def scene_id(self, scene_id):
		self._scene_id = scene_id

	@property
	def cover(self):
		return self._cover

	@cover.setter
	def cover(self, cover):
		self._cover = cover

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.scene_id, self.cover]
		else:
			return [self.scene_id, self.cover]

class ImagesRow(TableRow):
	def __init__(self):
		super().__init__('images')
		self._id = None
		self._path = None
		self._checksum = None
		self._title = None
		self._rating = None
		self._size = None
		self._width = None
		self._height = None
		self._studio_id = None
		self._o_counter = None
		self._created_at = None
		self._updated_at = None
		self._file_mod_time = None
		self._organized = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def id(self):
		return self._id

	@id.setter
	def id(self, id):
		self._id = id

	@property
	def path(self):
		return self._path

	@path.setter
	def path(self, path):
		self._path = path

	@property
	def checksum(self):
		return self._checksum

	@checksum.setter
	def checksum(self, checksum):
		self._checksum = checksum

	@property
	def title(self):
		return self._title

	@title.setter
	def title(self, title):
		self._title = title

	@property
	def rating(self):
		return self._rating

	@rating.setter
	def rating(self, rating):
		self._rating = rating

	@property
	def size(self):
		return self._size

	@size.setter
	def size(self, size):
		self._size = size

	@property
	def width(self):
		return self._width

	@width.setter
	def width(self, width):
		self._width = width

	@property
	def height(self):
		return self._height

	@height.setter
	def height(self, height):
		self._height = height

	@property
	def studio_id(self):
		return self._studio_id

	@studio_id.setter
	def studio_id(self, studio_id):
		self._studio_id = studio_id

	@property
	def o_counter(self):
		return self._o_counter

	@o_counter.setter
	def o_counter(self, o_counter):
		self._o_counter = o_counter

	@property
	def created_at(self):
		return self._created_at

	@created_at.setter
	def created_at(self, created_at):
		self._created_at = created_at

	@property
	def updated_at(self):
		return self._updated_at

	@updated_at.setter
	def updated_at(self, updated_at):
		self._updated_at = updated_at

	@property
	def file_mod_time(self):
		return self._file_mod_time

	@file_mod_time.setter
	def file_mod_time(self, file_mod_time):
		self._file_mod_time = file_mod_time

	@property
	def organized(self):
		return self._organized

	@organized.setter
	def organized(self, organized):
		self._organized = organized

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.id, self.path, self.checksum, self.title, self.rating, self.size, self.width, self.height, self.studio_id, self.o_counter, self.created_at, self.updated_at, self.file_mod_time, self.organized]
		else:
			return [self.path, self.checksum, self.title, self.rating, self.size, self.width, self.height, self.studio_id, self.o_counter, self.created_at, self.updated_at, self.file_mod_time, self.organized]

class PerformersImagesRow(TableRow):
	def __init__(self):
		super().__init__('performers_images')
		self._performer_id = None
		self._image_id = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def performer_id(self):
		return self._performer_id

	@performer_id.setter
	def performer_id(self, performer_id):
		self._performer_id = performer_id

	@property
	def image_id(self):
		return self._image_id

	@image_id.setter
	def image_id(self, image_id):
		self._image_id = image_id

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.performer_id, self.image_id]
		else:
			return [self.performer_id, self.image_id]

class ImagesTagsRow(TableRow):
	def __init__(self):
		super().__init__('images_tags')
		self._image_id = None
		self._tag_id = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def image_id(self):
		return self._image_id

	@image_id.setter
	def image_id(self, image_id):
		self._image_id = image_id

	@property
	def tag_id(self):
		return self._tag_id

	@tag_id.setter
	def tag_id(self, tag_id):
		self._tag_id = tag_id

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.image_id, self.tag_id]
		else:
			return [self.image_id, self.tag_id]

class SceneStashIdsRow(TableRow):
	def __init__(self):
		super().__init__('scene_stash_ids')
		self._scene_id = None
		self._endpoint = None
		self._stash_id = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def scene_id(self):
		return self._scene_id

	@scene_id.setter
	def scene_id(self, scene_id):
		self._scene_id = scene_id

	@property
	def endpoint(self):
		return self._endpoint

	@endpoint.setter
	def endpoint(self, endpoint):
		self._endpoint = endpoint

	@property
	def stash_id(self):
		return self._stash_id

	@stash_id.setter
	def stash_id(self, stash_id):
		self._stash_id = stash_id

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.scene_id, self.endpoint, self.stash_id]
		else:
			return [self.scene_id, self.endpoint, self.stash_id]

class PerformerStashIdsRow(TableRow):
	def __init__(self):
		super().__init__('performer_stash_ids')
		self._performer_id = None
		self._endpoint = None
		self._stash_id = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def performer_id(self):
		return self._performer_id

	@performer_id.setter
	def performer_id(self, performer_id):
		self._performer_id = performer_id

	@property
	def endpoint(self):
		return self._endpoint

	@endpoint.setter
	def endpoint(self, endpoint):
		self._endpoint = endpoint

	@property
	def stash_id(self):
		return self._stash_id

	@stash_id.setter
	def stash_id(self, stash_id):
		self._stash_id = stash_id

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.performer_id, self.endpoint, self.stash_id]
		else:
			return [self.performer_id, self.endpoint, self.stash_id]

class StudioStashIdsRow(TableRow):
	def __init__(self):
		super().__init__('studio_stash_ids')
		self._studio_id = None
		self._endpoint = None
		self._stash_id = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def studio_id(self):
		return self._studio_id

	@studio_id.setter
	def studio_id(self, studio_id):
		self._studio_id = studio_id

	@property
	def endpoint(self):
		return self._endpoint

	@endpoint.setter
	def endpoint(self, endpoint):
		self._endpoint = endpoint

	@property
	def stash_id(self):
		return self._stash_id

	@stash_id.setter
	def stash_id(self, stash_id):
		self._stash_id = stash_id

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.studio_id, self.endpoint, self.stash_id]
		else:
			return [self.studio_id, self.endpoint, self.stash_id]

class GalleriesRow(TableRow):
	def __init__(self):
		super().__init__('galleries')
		self._id = None
		self._path = None
		self._checksum = None
		self._zip = None
		self._title = None
		self._url = None
		self._date = None
		self._details = None
		self._studio_id = None
		self._rating = None
		self._file_mod_time = None
		self._organized = None
		self._created_at = None
		self._updated_at = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def id(self):
		return self._id

	@id.setter
	def id(self, id):
		self._id = id

	@property
	def path(self):
		return self._path

	@path.setter
	def path(self, path):
		self._path = path

	@property
	def checksum(self):
		return self._checksum

	@checksum.setter
	def checksum(self, checksum):
		self._checksum = checksum

	@property
	def zip(self):
		return self._zip

	@zip.setter
	def zip(self, zip):
		self._zip = zip

	@property
	def title(self):
		return self._title

	@title.setter
	def title(self, title):
		self._title = title

	@property
	def url(self):
		return self._url

	@url.setter
	def url(self, url):
		self._url = url

	@property
	def date(self):
		return self._date

	@date.setter
	def date(self, date):
		self._date = date

	@property
	def details(self):
		return self._details

	@details.setter
	def details(self, details):
		self._details = details

	@property
	def studio_id(self):
		return self._studio_id

	@studio_id.setter
	def studio_id(self, studio_id):
		self._studio_id = studio_id

	@property
	def rating(self):
		return self._rating

	@rating.setter
	def rating(self, rating):
		self._rating = rating

	@property
	def file_mod_time(self):
		return self._file_mod_time

	@file_mod_time.setter
	def file_mod_time(self, file_mod_time):
		self._file_mod_time = file_mod_time

	@property
	def organized(self):
		return self._organized

	@organized.setter
	def organized(self, organized):
		self._organized = organized

	@property
	def created_at(self):
		return self._created_at

	@created_at.setter
	def created_at(self, created_at):
		self._created_at = created_at

	@property
	def updated_at(self):
		return self._updated_at

	@updated_at.setter
	def updated_at(self, updated_at):
		self._updated_at = updated_at

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.id, self.path, self.checksum, self.zip, self.title, self.url, self.date, self.details, self.studio_id, self.rating, self.file_mod_time, self.organized, self.created_at, self.updated_at]
		else:
			return [self.path, self.checksum, self.zip, self.title, self.url, self.date, self.details, self.studio_id, self.rating, self.file_mod_time, self.organized, self.created_at, self.updated_at]

class ScenesGalleriesRow(TableRow):
	def __init__(self):
		super().__init__('scenes_galleries')
		self._scene_id = None
		self._gallery_id = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def scene_id(self):
		return self._scene_id

	@scene_id.setter
	def scene_id(self, scene_id):
		self._scene_id = scene_id

	@property
	def gallery_id(self):
		return self._gallery_id

	@gallery_id.setter
	def gallery_id(self, gallery_id):
		self._gallery_id = gallery_id

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.scene_id, self.gallery_id]
		else:
			return [self.scene_id, self.gallery_id]

class GalleriesImagesRow(TableRow):
	def __init__(self):
		super().__init__('galleries_images')
		self._gallery_id = None
		self._image_id = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def gallery_id(self):
		return self._gallery_id

	@gallery_id.setter
	def gallery_id(self, gallery_id):
		self._gallery_id = gallery_id

	@property
	def image_id(self):
		return self._image_id

	@image_id.setter
	def image_id(self, image_id):
		self._image_id = image_id

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.gallery_id, self.image_id]
		else:
			return [self.gallery_id, self.image_id]

class PerformersGalleriesRow(TableRow):
	def __init__(self):
		super().__init__('performers_galleries')
		self._performer_id = None
		self._gallery_id = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def performer_id(self):
		return self._performer_id

	@performer_id.setter
	def performer_id(self, performer_id):
		self._performer_id = performer_id

	@property
	def gallery_id(self):
		return self._gallery_id

	@gallery_id.setter
	def gallery_id(self, gallery_id):
		self._gallery_id = gallery_id

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.performer_id, self.gallery_id]
		else:
			return [self.performer_id, self.gallery_id]

class GalleriesTagsRow(TableRow):
	def __init__(self):
		super().__init__('galleries_tags')
		self._gallery_id = None
		self._tag_id = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def gallery_id(self):
		return self._gallery_id

	@gallery_id.setter
	def gallery_id(self, gallery_id):
		self._gallery_id = gallery_id

	@property
	def tag_id(self):
		return self._tag_id

	@tag_id.setter
	def tag_id(self, tag_id):
		self._tag_id = tag_id

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.gallery_id, self.tag_id]
		else:
			return [self.gallery_id, self.tag_id]

class PerformersTagsRow(TableRow):
	def __init__(self):
		super().__init__('performers_tags')
		self._performer_id = None
		self._tag_id = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def performer_id(self):
		return self._performer_id

	@performer_id.setter
	def performer_id(self, performer_id):
		self._performer_id = performer_id

	@property
	def tag_id(self):
		return self._tag_id

	@tag_id.setter
	def tag_id(self, tag_id):
		self._tag_id = tag_id

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.performer_id, self.tag_id]
		else:
			return [self.performer_id, self.tag_id]

class TagAliasesRow(TableRow):
	def __init__(self):
		super().__init__('tag_aliases')
		self._tag_id = None
		self._alias = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def tag_id(self):
		return self._tag_id

	@tag_id.setter
	def tag_id(self, tag_id):
		self._tag_id = tag_id

	@property
	def alias(self):
		return self._alias

	@alias.setter
	def alias(self, alias):
		self._alias = alias

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.tag_id, self.alias]
		else:
			return [self.tag_id, self.alias]

class SavedFiltersRow(TableRow):
	def __init__(self):
		super().__init__('saved_filters')
		self._id = None
		self._name = None
		self._mode = None
		self._filter = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def id(self):
		return self._id

	@id.setter
	def id(self, id):
		self._id = id

	@property
	def name(self):
		return self._name

	@name.setter
	def name(self, name):
		self._name = name

	@property
	def mode(self):
		return self._mode

	@mode.setter
	def mode(self, mode):
		self._mode = mode

	@property
	def filter(self):
		return self._filter

	@filter.setter
	def filter(self, filter):
		self._filter = filter

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.id, self.name, self.mode, self.filter]
		else:
			return [self.name, self.mode, self.filter]

class TagsRelationsRow(TableRow):
	def __init__(self):
		super().__init__('tags_relations')
		self._parent_id = None
		self._child_id = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def parent_id(self):
		return self._parent_id

	@parent_id.setter
	def parent_id(self, parent_id):
		self._parent_id = parent_id

	@property
	def child_id(self):
		return self._child_id

	@child_id.setter
	def child_id(self, child_id):
		self._child_id = child_id

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.parent_id, self.child_id]
		else:
			return [self.parent_id, self.child_id]

class StudioAliasesRow(TableRow):
	def __init__(self):
		super().__init__('studio_aliases')
		self._studio_id = None
		self._alias = None

	@property
	def table_name(self):
		return self._table_name

	@property
	def studio_id(self):
		return self._studio_id

	@studio_id.setter
	def studio_id(self, studio_id):
		self._studio_id = studio_id

	@property
	def alias(self):
		return self._alias

	@alias.setter
	def alias(self, alias):
		self._alias = alias

	def __str__(self):
		return str(self.__class__) + ": " + str(self.__dict__)

	def values_list(self, include_id=False):
		if include_id:
			return [self.studio_id, self.alias]
		else:
			return [self.studio_id, self.alias]




import sys


# Log messages sent from a plugin instance are transmitted via stderr and are
# encoded with a prefix consisting of special character SOH, then the log
# level (one of t, d, i, w, e, or p - corresponding to trace, debug, info,
# warning, error and progress levels respectively), then special character
# STX.
#
# The LogTrace, LogDebug, LogInfo, LogWarning, and LogError methods, and their equivalent
# formatted methods are intended for use by plugin instances to transmit log
# messages. The LogProgress method is also intended for sending progress data.
#

def __prefix(level_char):
	start_level_char = b'\x01'
	end_level_char = b'\x02'

	ret = start_level_char + level_char + end_level_char
	return ret.decode()


def log(level_char, s):
	if level_char == "":
		return

	print(__prefix(level_char) + s + "\n", file=sys.stderr, flush=True)


def LogTrace(s):
	log(b't', s)


def LogDebug(s):
	log(b'd', s)


def LogInfo(s):
	log(b'i', s)


def LogWarning(s):
	log(b'w', s)


def LogError(s):
	log(b'e', s)


def LogProgress(p):
	progress = min(max(0, p), 1)
	log(b'p', str(progress))